# Copyright (c) 2021 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the COPYING file.

from cvprac.cvp_client import CvpClient
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import requests.packages.urllib3
requests.packages.urllib3.disable_warnings()

# Create connection to CloudVision
clnt = CvpClient()
clnt.connect(nodes=['cvp1'], username="username",password="password")

# Modify existing configlet

configletName = "cvprac_example"

configlet = """!
interface Ethernet10
   description DUB_R04
   ip address 10.144.144.2/24
!
"""

configletID = clnt.api.get_configlet_by_name(configletName)['key']

clnt.api.update_configlet( configlet, configletID, configletName)
